@@ -1,31 +1,48 @@
-use list_fn::{FlatMap, FlatMapFn, FlatMapList, ListFn, ListState, ResultFn, ListSome};
+use list_fn::{FlatMap, FlatMapFn, FlatMapList, ListFn, ListSome, ListState, ResultFn};
 use std::marker::PhantomData;
 use uints::UInt;
 
-pub struct Lsb0List<T: UInt> {
-    value: T,
-    size: u8,
+// LSB first bit vector.
+#[derive(Clone, Copy)]
+pub struct BitVec<T: UInt> {
+    pub array: T,
+    pub size: u8,
 }
 
-impl<T: UInt> Lsb0List<T> {
-    fn new(value: T) -> Self {
-        Lsb0List {
-            value,
-            size: T::BITS,
+impl<T: UInt> BitVec<T> {
+    pub fn new_full(array: T) -> Self {
+        Self::new(array, T::BITS)
+    }
+    pub fn new(array: T, size: u8) -> Self {
+        BitVec { array, size }
+    }
+    pub fn concat(self, v: Self) -> Self {
+        BitVec {
+            array: self.array | (v.array << self.size),
+            size: self.size + v.size,
         }
     }
 }
 
-impl<T: UInt> ListFn for Lsb0List<T> {
+impl<T: UInt> Default for BitVec<T> {
+    fn default() -> Self {
+        BitVec {
+            array: T::ZERO,
+            size: 0,
+        }
+    }
+}
+
+impl<T: UInt> ListFn for BitVec<T> {
     type Item = bool;
     type End = ();
     fn next(self) -> ListState<Self> {
         match self.size {
             0 => ListState::End(()),
             size => ListState::Some(ListSome {
-                first: self.value & T::ONE != T::ZERO,
-                next: Lsb0List {
-                    value: self.value >> 1,
+                first: self.array & T::ONE != T::ZERO,
+                next: BitVec {
+                    array: self.array >> 1,
                     size: size - 1,
                 },
             }),
@@ -43,9 +60,9 @@ impl<T: UInt> Default for Lsb0FlatMap<T> {
 
 impl<T: UInt> FlatMapFn for Lsb0FlatMap<T> {
     type Input = T;
-    type OutputList = Lsb0List<T>;
-    fn map(&self, item: T) -> Lsb0List<T> {
-        Lsb0List::new(item)
+    type OutputList = BitVec<T>;
+    fn map(&self, item: T) -> BitVec<T> {
+        BitVec::new_full(item)
     }
 }
 
